@@ -17,13 +17,19 @@ use URI;
 
 Log::Log4perl::Appender::Elasticsearch - implements appending to Elasticsearch
 
+=head1 DESCRIPTION
+
+This is a simple appender for writing log entries to Elasticsearch via L<Index API|http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/docs-index_.html>.
+
+L<Log::Log4perl::Appender::Elasticsearch::Bulk> does the same task by using L<Bulk API|http://www.elasticsearch.org/guide/en/elasticsearch/reference/current/docs-bulk.html>.
+
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+$Log::Log4perl::Appender::Elasticsearch::VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -67,31 +73,43 @@ our $VERSION = '0.04';
 
 =over 4
 
-=item nodes
+=item
+
+nodes
 
 a comma separeted list of nodes. The message will be sent to the next node only if previous request failed
 
-=item index
+=item
+
+index
 
 The name of the elasticsearch index the message will be stored in.
 
-=item type
+=item
+
+type
 
 The name of the type in given index the message belongs to.
 
-=item use_https
+=item
+
+use_https
 
 0|1 global https setting for all nodes
 
 the individual https setting possible too: C<log4perl.appender.ES.nodes = https://user:password@node1:9200,localhost:9200>
 
-=item user_agent
+=item
+
+user_agent
 
 LWP::UserAgent parameters.
 
 C<log4perl.appender.ES.user_agent.timeout = 5>
 
-=item headers
+=item
+
+headers
 
 HTTP::Headers parameters
 
@@ -112,7 +130,7 @@ sub new {
 
 sub log {
     my ($self, %p) = @_;
-    $self->_send_request($self->_prepare_body(%p));
+    $self->_send_request($self->_to_json($self->_prepare_body(%p)));
 }
 
 sub _init {
@@ -163,13 +181,13 @@ sub _init {
 } ## end sub _init
 
 sub _send_request {
-    my ($self, $b) = @_;
+    my ($self, $data, $suffix) = @_;
     my @nodes = @{ $self->{_nodes} };
     my (@errors, $ok);
     do {
         my $node = shift @nodes;
-        my $uri  = $self->_uri($node);
-        my $req  = $self->_request($uri, $b);
+        my $uri  = $self->_uri($node, $suffix);
+        my $req  = $self->_request($uri, $data);
 
         my $resp = $self->{_user_agent}->request($req);
         $ok = $resp->is_success;
@@ -183,10 +201,10 @@ sub _send_request {
 } ## end sub _send_request
 
 sub _uri {
-    my ($self, $node) = @_;
+    my ($self, $node, $suffix) = @_;
     my $uri = $node->clone;
     $uri->path(join '', $uri->path,
-        join('/', $self->{_index}, $self->{_type}, ''));
+        join('/', $self->{_index}, $self->{_type}, $suffix ? $suffix : ''));
 
     return $uri;
 } ## end sub _uri
@@ -207,9 +225,7 @@ sub _headers {
 } ## end sub _headers
 
 sub _request {
-    my ($self, $uri, $b) = @_;
-
-    my $data = encode_json($b);
+    my ($self, $uri, $data) = @_;
 
     return HTTP::Request->new(
         POST => $uri,
@@ -237,46 +253,17 @@ sub _prepare_body {
     return $b;
 } ## end sub _prepare_body
 
+sub _to_json {
+    my ($self, $o) = @_;
+    return JSON::encode_json($o);
+}
+
+1;    # End of Log::Log4perl::Appender::Elasticsearch
+__END__
+
 =head1 AUTHOR
 
-Alexei Pastuchov C<< <palik at cpan.com> >>
-
-=head1 BUGS
-
-Please report any bugs or feature requests to C<bug-log-log4perl-appender-elasticsearch at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Log-Log4perl-Appender-Elasticsearch>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
-
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Log::Log4perl::Appender::Elasticsearch
-
-
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker (report bugs here)
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Log-Log4perl-Appender-Elasticsearch>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/Log-Log4perl-Appender-Elasticsearch>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/Log-Log4perl-Appender-Elasticsearch>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/Log-Log4perl-Appender-Elasticsearch/>
-
-=back
-
+Alexei Pastuchov C<< <palik at cpan.org> >>
 
 =head1 REPOSITORY
 
@@ -284,11 +271,9 @@ L<https://github.com/p-alik/Log-Log4perl-Appender-Elasticsearch.git>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2014 by Alexei Pastuchov E<lt>palik at cpan.comE<gt>.
+Copyright 2014, 2015 by Alexei Pastuchov E<lt>palik at cpan.orgE<gt>.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
-
-1;    # End of Log::Log4perl::Appender::Elasticsearch
